/*
    \brief Deklarationen für die Funktionen der Setupdatensätze.

    \author Bernd Ottmann
    \date   21.02.2003
*/
#pragma once
#ifndef DATAFOX_SETUP_RECORDS_H
#ifndef DOXYGEN_SHOULD_SKIP_THIS
#define DATAFOX_SETUP_RECORDS_H
#endif

#include "df_defines.h"

/*!
    \brief Datensätze löschen.

    Bei den verwendeten Ringspeichern werden die Datenzeiger zurückgesetzt. Die im Gerät eventuell enthaltenen
    Datensätze werden dadurch unwiederruflich gelöscht.

    \ingroup setup_records

    \export{_DFCComClearData@8, DFCComClearData}

    \param[in] connectionId Zu verwendende Verbindungs-Id.
    \param[in] deviceAddress Anzusprechende Geräteadresse.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCComClearData( int connectionId, int deviceAddress );

/*!
    \brief Datensätze aus Gerät lesen.

    Es wird ein Datensatz aus dem Gerät ausgelesen und in einem Bibliothekspuffer zwischengespeichert.
    Zum weiteren Abruf der Datensätze wird die Funktion DFCComGetDatensatz() verwendet.

    \ingroup setup_records

    \export{_DFCComCollectData@12, DFCComCollectData}

    \deprecated Bitte verwenden Sie die Funktionen DFCReadRecord() und DFCQuitRecord() zum Auslesen der
    Datensätze.

    \param[in] connectionId Zu verwendende Verbindungs-Id.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval -1 Fehler bei Ausführung.
    \retval 0 Keine Datensätze vorhanden.
    \retval 1 Es wurde ein Offline-Datensatz gelesen.
    \retval 2 Es wurde ein [Online-Datensatz](\ref dfc_records_online) gelesen.
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCComCollectData( int connectionId, int deviceAddress, int *errorNumber );

/*!
    \brief Datensatz aus Bibliothekspuffer lesen.

    Es wird ein Datensatz aus dem Bibliothekspuffer übernommen und ggf. ein weiterer aus dem Gerät gelesen.
    Die Geräteadresse ist durch vorherigen Aufruf der Funktion DFCComCollectData() bekannt, diese muss einmalig
    aufgerufen worden sein um mit DFCComGetDatensatz Datensätze abrufen zu können.

    \ingroup setup_records

    \export{_DFCComGetDatensatz@16, DFCComGetDatensatz}

    \deprecated Bitte verwenden Sie die Funktionen DFCReadRecord() und DFCQuitRecord() zum Auslesen der
    Datensätze.

    \param[in] connectionId Zu verwendende Verbindungs-Id.
    \param[out] buffer Datenpuffer.
    \param[in,out] bufferSize Als [in] die reservierte Größe von \c buffer, als [out] die übergebene Datensatzlänge.
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval -1 Fehler bei Ausführung.
    \retval 0 Keine weiteren Datensätze vorhanden.
    \retval 1 Es wurde ein Offline-Datensatz übergeben.
    \retval 2 Es wurde ein [Online-Datensatz](\ref dfc_records_online) übergeben. Auch letzter Datensatz.
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCComGetDatensatz( int connectionId, unsigned char *buffer, int *bufferSize, int *errorNumber );

/*!
    \brief Aufbau der Datensatztabellen aus dem Geräte lesen.

    Es werden die im Gerätesetup verwendeten Tabellenbeschreibungen für die Datensätze in den Bibliotheksinternen
    Puffer gelesen. Hiernach kann mit den Funktionen DFCDatBCnt(), DFCDatBDatensatz() und DFCDatBFeld()
    auf die ermittelten Daten zugegriffen werden.

    \ingroup setup_records

    \export{_DFCLoadDatensatzbeschreibung@12, DFCLoadDatensatzbeschreibung}

    \param[in] connectionId Zu verwendende Verbindungs-Id.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCLoadDatensatzbeschreibung( int connectionId, int deviceAddress, int *errorNumber );

/*!
    \brief Anzahl der gelesenen Datensatztabellen abrufen.

    Auf Grundlage der zuvor mit DFCLoadDatensatzbeschreibung() gelesenen Datensatztabellen, wird
    die im Gerätesetup definierte Anzahl zurückgegeben.

    \ingroup setup_records

    \export{_DFCDatBCnt@4, DFCDatBCnt}

    \param[in] connectionId Zu verwendende Verbindungs-Id.

    \return Anzahl der Datensatztabellen.
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCDatBCnt( int connectionId );

/*!
    \brief Grunddaten einer Datensatztabelle abrufen.

    Auf Grundlage der zuvor mit DFCLoadDatensatzbeschreibung() gelesenen Datensatztabellen, werden
    die Grunddaten zu einer Datensatztabelle zurückgegeben.

    \ingroup setup_records

    \export{_DFCDatBDatensatz@16, DFCDatBDatensatz}

    \param[in] connectionId Zu verwendende Verbindungs-Id.
    \param[in] number Datensatznummer. Bereich 0 bis DFCDatBCnt() - 1
    \param[out] name Zeichenpuffer für den Namen. Muss mindestens 17 Zeichen lang sein.
    \param[out] fieldCount Anzahl der Felder.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCDatBDatensatz( int connectionId, int number, char *name, int *fieldCount );

/*!
    \brief Grunddaten eines Datensatzfeldes abrufen.

    Auf Grundlage der zuvor mit DFCLoadDatensatzbeschreibung() gelesenen Datensatztabellen, werden
    die Grunddaten zu einem Datensatzfeld zurückgegeben.

    \ingroup setup_records

    \export{_DFCDatBFeld@24, DFCDatBFeld}

    \param[in] connectionId Zu verwendende Verbindungs-Id.
    \param[in] datBNumber Datensatznummer. Bereich 0 bis DFCDatBCnt() - 1.
    \param[in] number Feldnummer. Bereich 0 bis maximal 25.
    \param[out] name Zeichenpuffer für den Feldnamen. Muss mindestens 17 Zeichen lang sein.
    \param[out] type Datentyp.
    <table class="paramvalues">
        <tr><th>Wert</th><th>Bedeutung</th></tr>
        <tr><td>1</td><td>unsigned long 4 Bytes</td></tr>
        <tr><td>2</td><td>Date und Time 7 Bytes</td></tr>
        <tr><td>3</td><td>Wie Typ 4, es sind jedoch nur Ziffern erlaubt</td></tr>
        <tr><td>4</td><td>Alpha-Numerisch ASCII</td></tr>
        <tr><td>5</td><td>Datum 4 Bytes</td></tr>
        <tr><td>6</td><td>Zeit 3 Bytes</td></tr>
    </table>
    \param[out] size Länge. Die Längenangabe beinhaltet ein eventuell benötigtes Nullterminierungszeichen bei Feldern
    für Zeichenfolgen.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCDatBFeld( int connectionId, int datBNumber, int number, char *name, int *type, int *size );

/*!
    \brief Datensatz aus Gerät lesen.

    Es wird versucht einen Datensatz aus dem Gerät zu lesen. Nach erfolgreicher Vararbeitung muss dieser mit
    DFCQuitRecord() quittiert werden, damit weitere Datensätze abgerufen werden können. Wird ein Datensatz nicht
    mit DFCQuitRecord quittiert wird er als ein Wiederholt gelieferter Datensatz gemeldet.

    \ingroup setup_records

    \export{_DFCReadRecord@20, DFCReadRecord}

    \param[in] connectionId Zu verwendende Verbindungs-Id.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[out] buffer Datenpuffer.
    \param[in,out] bufferSize Als [in] die reservierte Größe von \c buffer, als [out] die übergebene Datensatzlänge.
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval -1 Fehler bei Ausführung.
    \retval 0 Keine Datensätze vorhanden.
    \retval 1 [Online-Datensatz](\ref dfc_records_online) gelesen.
    \retval 2 Wiederholt ein Online-Datensatz gelesen.
    \retval 3 Offline-Datensatz gelesen.
    \retval 4 Wiederholt ein Offline-Datensatz gelesen.
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCReadRecord( int connectionId, int deviceAddress, unsigned char *buffer, int *bufferSize, int *errorNumber );

/*!
    \brief Datensatz im Gerät quittieren.

    Es wird versucht einen zuvor mit DFCReadRecord() gelesenen Datensatz zu quittieren. Nach erfolgreicher Quittierung
    kann mit einem folgenden Aufruf von DFCReadRecord der ggf. nächste Datensatz gelesen werden.

    \ingroup setup_records

    \export{_DFCQuitRecord@12, DFCQuitRecord}

    \param[in] connectionId Zu verwendende Verbindungs-Id.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval -1 Fehler bei Ausführung.
    \retval 0 Keine Quittung benötigt.
    \retval 1 Online-Datensatz quittiert.
    \retval 2 Wiederholt einen Online-Datensatz quittiert.
    \retval 3 Offline-Datensatz quittiert.
    \retval 4 Wiederholt einen Offline-Datensatz quittiert.
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCQuitRecord( int connectionId, int deviceAddress, int *errorNumber );

#if (DF_API_VERSION >= 0x04010600)

/*!
    \brief Datensätze wiederherstellen.

    Durch ein Zurücksetzen der Datensatzzeiger auf den letzten auslesbaren Datensatz wird ein Wiederauslesen über
    DFCReadRecord() möglich. Durch Verwendung eines Ringspeichers in den Geräten werden ggf. ältere Datensätze durch
    aktuellere überschrieben.

    \ingroup setup_records

    \export{_DFCRestoreRecords@12, DFCRestoreRecords}

    \warning Durch ein Wiederherstellen der Datensätze kann eine ggf. sehr große Datensatzmenge entstehen.

    \note Durch Aufruf von DFCComClearData() oder DFCSetupLaden() werden die Datensätze gelöscht. Führen Sie also
    jewils am Monatsende DFCComClearData nach Abruf der Datensätze aus, können Sie mit Hilfe dieser Funktion
    eine Art Monatsbackup auf den Geräten realisieren.

    \param[in] connectionId Zu verwendende Verbindungs-Id.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.

    \since 04.01.06 enthalten
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCRestoreRecords( int connectionId, int deviceAddress, int *errorNumber );

#endif

// ########################################################
// #############* PRIVATE FUNKTIONEN* #################
// ########################################################

// Funktionen sind nicht dokumentiert.
// Sie werden von AESetup oder Talk verwendet.
// Unterliegen evntl. Aenderungen.

#if defined( USE_PRIVATE )

//    \export{_DFCCheckData@12, DFCCheckData}
DF_API_EXTERN_DECORATE int DF_API_CALL DFCCheckData( int connectionId, int deviceAddress, int *errorNumber );

#endif // USE_PRIVATE

#endif
